Attribute VB_Name = "modNetwork"
Option Explicit

Private Type USER_INFO
    Name As String
    Comment As String
    UserComment As String
    FullName As String
End Type

Private Type USER_INFO_API
    Name As Long
    Comment As Long
    UserComment As Long
    FullName As Long
End Type


Private Declare Function NetUserEnum Lib "netapi32" _
  (lpServer As Any, ByVal Level As Long, _
   ByVal Filter As Long, lpBuffer As Long, _
   ByVal PrefMaxLen As Long, EntriesRead As Long, _
   TotalEntries As Long, ResumeHandle As Long) As Long
   
Private Declare Function NetApiBufferFree Lib "netapi32" _
   (ByVal pBuffer As Long) As Long

Private Declare Sub CopyMem Lib "KERNEL32" Alias _
   "RtlMoveMemory" (pTo As Any, uFrom As Any, _
    ByVal lSize As Long)
    
Private Declare Function lstrlenW Lib "KERNEL32" _
 (ByVal lpString As Long) As Long

Private Const NERR_Success As Long = 0&
Private Const ERROR_MORE_DATA As Long = 234&

Private Const FILTER_TEMP_DUPLICATE_ACCOUNT As Long = &H1&
Private Const FILTER_NORMAL_ACCOUNT As Long = &H2&
Private Const FILTER_PROXY_ACCOUNT As Long = &H4&
Private Const FILTER_INTERDOMAIN_TRUST_ACCOUNT As Long = &H8&
Private Const FILTER_WORKSTATION_TRUST_ACCOUNT As Long = &H10&
Private Const FILTER_SERVER_TRUST_ACCOUNT As Long = &H20&

Private Declare Function NetGetDCName Lib "NETAPI32.DLL" _
  (strServerName As Any, strDomainName As Any, _
  pBuffer As Long) As Long

Private Declare Sub MoveMemory Lib "KERNEL32" Alias _
   "RtlMoveMemory" (hpvDest As Any, _
   ByVal hpvSource As Long, ByVal cbCopy As Long)


Private Declare Function lstrcpyW Lib "KERNEL32" _
  (lpString1 As Byte, ByVal lpString2 As Long) As Long

Private Declare Function MultiByteToWideChar Lib "KERNEL32" _
   (ByVal CodePage As Long, ByVal dwFlags As Long, _
   ByVal lpMultiByteStr As String, _
   ByVal cchMultiByte As Long, ByVal lpWideCharStr As String, _
   ByVal cchWideChar As Long) As Long

Private Const CP_ACP = 0

Private Type NetResource
   dwScope                             As Long
   dwType                              As Long
   dwDisplayType                       As Long
   dwUsage                             As Long
   pLocalName                          As Long
   pRemoteName                         As Long
   pComment                            As Long
   pProvider                           As Long
End Type

Private Declare Function WNetOpenEnum _
   Lib "mpr.dll" Alias "WNetOpenEnumA" _
   (ByVal dwScope As Long, _
   ByVal dwType As Long, _
   ByVal dwUsage As Long, _
   lpNetResource As Any, _
   lppEnumHwnd As Long) As Long

Private Declare Function WNetEnumResource _
   Lib "mpr.dll" Alias "WNetEnumResourceA" _
   (ByVal pEnumHwnd As Long, _
   lpcCount As Long, _
   lpBuffer As NetResource, _
   lpBufferSize As Long) As Long

Private Declare Function WNetCloseEnum _
   Lib "mpr.dll" _
   (ByVal p_lngEnumHwnd As Long) As Long

Private Declare Function NetUserGetInfo _
   Lib "NETAPI32.DLL" _
   (ServerName As Byte, _
   Username As Byte, _
   ByVal Level As Long, _
   Buffer As Long) As Long
   
Private Declare Function StrLenA _
   Lib "KERNEL32" Alias "lstrlenA" _
   (ByVal Ptr As Long) As Long
   
Private Declare Function StrCopyA _
   Lib "KERNEL32" Alias "lstrcpyA" _
   (ByVal RetVal As String, _
   ByVal Ptr As Long) As Long

Private Const MAX_RESOURCES            As Long = 256
Private Const RESOURCE_GLOBALNET       As Long = &H2&
Private Const RESOURCETYPE_ANY         As Long = &H0&
Private Const RESOURCEUSAGE_ALL        As Long = &H0&
Private Const NO_ERROR                 As Long = 0&
Private Const RESOURCE_ENUM_ALL        As Long = &HFFFF

Private Declare Function NetMessageBufferSend Lib _
  "NETAPI32.DLL" (yServer As Any, yToName As Byte, _
  yFromName As Any, yMsg As Byte, ByVal lSize As Long) As Long

Public Function GetUsers(UserNames() As String, _
   Optional ServerName As String = "") As Boolean
    
    'PURPOSE: Get LoginNames of all users on the domain and
    'save in a string array
    
    'PARAMETERS: UserNames(): Empty String array, passed byref,
      'to hold user names
    
    'ServerName (Optional): Set to "" if you want user
      'names for current machine, otherwise, set to the server
      'you want (e.g., Domain Controller Name)
    
    'RETURNS: True if successful, false otherwise
    
    'EXAMPLE:
        'Dim sUsers() As String
        'dim lCtr as long
        'GetUsers sUsers, "MyDomainController"
        
        'OR FOR LOCAL MACHINE
        
        'GetUsers sUsers
   
    'For lCtr = LBound(sUsers) To UBound(sUsers)
     '   Debug.Print sUsers(lCtr)
    'Next
    
     'NOTES: WINDOWS NT/2000 only (hasn't been tested on 2000)
    
    Dim lptrStrBuffer As Long
    Dim lRet As Long
    Dim lUsersRead As Long
    Dim lTotalUsers As Long
    Dim lHnd As Long
    Dim etUserInfo As USER_INFO_API
    Dim bytServerName() As Byte
    Dim lElement As Long
    Dim Users() As USER_INFO
  
    ReDim Users(0) As USER_INFO
    ReDim UserNames(0) As String
    
    If Trim(ServerName) = "" Then
        'Local users
        bytServerName = vbNullString
    Else
        'Check the syntax of the ServerName string
        If InStr(ServerName, "\\") = 1 Then
            bytServerName = ServerName & vbNullChar
        Else
            bytServerName = "\\" & ServerName & vbNullChar
        End If
    End If
    lHnd = 0
    Do
        'Begin enumerating users
        If Trim(ServerName) = "" Then
            lRet = NetUserEnum(vbNullString, 10, _
             FILTER_NORMAL_ACCOUNT, lptrStrBuffer, 1, _
              lUsersRead, lTotalUsers, lHnd)
        Else
            lRet = NetUserEnum(bytServerName(0), 10, _
             FILTER_NORMAL_ACCOUNT, lptrStrBuffer, 1, _
               lUsersRead, lTotalUsers, lHnd)
        End If
        'Populate UserInfo Structure
        If lRet = ERROR_MORE_DATA Then
            If UserNames(0) = "" Then
                lElement = 0
            Else
                lElement = UBound(UserNames) + 1
            End If
            ReDim Preserve UserNames(lElement)
            ReDim Preserve Users(lElement) As USER_INFO
            
            CopyMem etUserInfo, ByVal lptrStrBuffer, _
               Len(etUserInfo)
            'data of interest
            Users(lElement).Name = PtrToString(etUserInfo.Name)

            'Other stuff you can get, but not
            'returned by this function
            'modify this function if you are interested

            Users(lElement).Comment = _
                  PtrToString(etUserInfo.Comment)
            Users(lElement).UserComment = _
                 PtrToString(etUserInfo.UserComment)
            Users(lElement).FullName = _
                 PtrToString(etUserInfo.FullName)

            UserNames(lElement) = Users(lElement).Name
        
        End If
        If lptrStrBuffer Then
            Call NetApiBufferFree(lptrStrBuffer)
        End If
        DoEvents
    Loop While lRet = ERROR_MORE_DATA
    GetUsers = True
    Exit Function
ErrHandler:
On Error Resume Next
Call NetApiBufferFree(lptrStrBuffer)
End Function

Private Function PtrToString(lpString As Long) As String
    'Convert a windows pointer to a VB string
    Dim bytBuffer() As Byte
    Dim lLen As Long
    
    If lpString Then
        lLen = lstrlenW(lpString) * 2
        If lLen Then
            ReDim bytBuffer(0 To (lLen - 1)) As Byte
            CopyMem bytBuffer(0), ByVal lpString, lLen
            PtrToString = bytBuffer
        End If
    End If
End Function

Public Function GetDC() As String
   ' Gets the name of the PDC for the machine on which the
   'program runs
   
   Dim szServer    As String
   Dim ptmpBuffer  As Long
   Dim sByte()     As Byte
   Dim lGotNameOK  As Long
   Dim lBufferOK   As Long
   
   On Error GoTo ErrHandler
   
   '# this will return nothing if the machine is not in a domain
   lGotNameOK = NetGetDCName(vbNullString, vbNullString, _
       ptmpBuffer)
   
   If lGotNameOK = 0 Then ' success
      
      ReDim sByte(256)
      
      ' ptmpbuffer is a pointer so copy contents using API call
      MoveMemory sByte(0), ptmpBuffer, 256
      
      If lBufferOK = 0 Then
         ' strip off trailing rubbish
         szServer = sByte
         szServer = szServer & vbNullChar
         GetDC = Left$(szServer, InStr(szServer, vbNullChar) - 1)
      End If
       ' free ptmpbuffer - not in other samples but
          'mentioned in documentation
      lBufferOK = NetApiBufferFree(ptmpBuffer)
   Else
      GetDC = ""
   End If
   Exit Function
ErrHandler:
    GetDC = ""
End Function

Public Sub GetDomains(lst As Object)

Dim p_avntDomains                   As Variant
Dim p_lngLoop                       As Long
Dim p_lngNumItems                   As Long

p_avntDomains = EnumDomains()

On Error Resume Next
p_lngNumItems = UBound(p_avntDomains)
On Error GoTo 0

If p_lngNumItems > 0 Then
   For p_lngLoop = 1 To p_lngNumItems
      lst.AddItem p_avntDomains(p_lngLoop)
   Next p_lngLoop
End If

End Sub

Public Function EnumDomains() As Variant

Dim p_lngRtn                        As Long
Dim p_lngEnumHwnd                   As Long
Dim p_lngCount                      As Long
Dim p_lngLoop                       As Long
Dim p_lngBufSize                    As Long
Dim p_astrDomainNames()             As String
Dim p_atypNetAPI(0 To MAX_RESOURCES) As NetResource

' ------------------------------------------
' First time thru, we are just getting the root level
' ------------------------------------------
p_lngEnumHwnd = 0&
p_lngRtn = WNetOpenEnum(dwScope:=RESOURCE_GLOBALNET, _
   dwType:=RESOURCETYPE_ANY, _
   dwUsage:=RESOURCEUSAGE_ALL, _
   lpNetResource:=ByVal 0&, _
   lppEnumHwnd:=p_lngEnumHwnd)

If p_lngRtn = NO_ERROR Then
   p_lngCount = RESOURCE_ENUM_ALL

   p_lngBufSize = UBound(p_atypNetAPI) * Len(p_atypNetAPI(0))
   p_lngRtn = WNetEnumResource(pEnumHwnd:=p_lngEnumHwnd, _
      lpcCount:=p_lngCount, _
      lpBuffer:=p_atypNetAPI(0), _
      lpBufferSize:=p_lngBufSize)

End If

If p_lngEnumHwnd <> 0 Then
   Call WNetCloseEnum(p_lngEnumHwnd)
End If

' ------------------------------------------
' Now we are going for the second level,
'     which should contain the domain names
' ------------------------------------------
p_lngRtn = WNetOpenEnum(dwScope:=RESOURCE_GLOBALNET, _
   dwType:=RESOURCETYPE_ANY, _
   dwUsage:=RESOURCEUSAGE_ALL, _
   lpNetResource:=p_atypNetAPI(0), _
   lppEnumHwnd:=p_lngEnumHwnd)

If p_lngRtn = NO_ERROR Then
   p_lngCount = RESOURCE_ENUM_ALL

   p_lngBufSize = UBound(p_atypNetAPI) * Len(p_atypNetAPI(0))
   p_lngRtn = WNetEnumResource(pEnumHwnd:=p_lngEnumHwnd, _
      lpcCount:=p_lngCount, _
      lpBuffer:=p_atypNetAPI(0), _
      lpBufferSize:=p_lngBufSize)

   If p_lngCount > 0 Then
      ReDim p_astrDomainNames(1 To p_lngCount) As String
      For p_lngLoop = 0 To p_lngCount - 1
         p_astrDomainNames(p_lngLoop + 1) = _
         PointerToAsciiStr(p_atypNetAPI(p_lngLoop).pRemoteName)
      Next p_lngLoop
   End If
End If

If p_lngEnumHwnd <> 0 Then
   Call WNetCloseEnum(p_lngEnumHwnd)
End If

' ------------------------------------------
' Set the return value
' ------------------------------------------
EnumDomains = p_astrDomainNames

End Function

Private Function PointerToAsciiStr(ByVal xi_lngPtrToString _
  As Long) As String

On Error Resume Next         ' Don't accept an error here

Dim p_lngLen                        As Long
Dim p_strStringValue                As String
Dim p_lngNullPos                    As Long
Dim p_lngRtn                        As Long

p_lngLen = StrLenA(xi_lngPtrToString)
If xi_lngPtrToString > 0 And p_lngLen > 0 Then
   p_strStringValue = Space$(p_lngLen + 1)
   p_lngRtn = StrCopyA(p_strStringValue, xi_lngPtrToString)
   p_lngNullPos = InStr(p_strStringValue, Chr$(0))
   If p_lngNullPos > 0 Then
      PointerToAsciiStr = Left$(p_strStringValue, _
         p_lngNullPos - 1) 'Lose the null terminator...
   Else
      'Just pass the string...
      PointerToAsciiStr = p_strStringValue
   End If
Else
   PointerToAsciiStr = ""
End If

End Function

Public Function BroadcastMessage(sToUser As String, _
   sFromUser As String, sMessage As String) As Boolean
 
   Dim yToName() As Byte
   Dim yFromName() As Byte
   Dim yMsg() As Byte
   Dim l As Long

   yToName = sToUser & vbNullChar
   yFromName = sFromUser & vbNullChar
   yMsg = sMessage & vbNullChar

   If NetMessageBufferSend(ByVal 0&, yToName(0), ByVal 0&, _
        yMsg(0), UBound(yMsg)) = NERR_Success Then
 
     BroadcastMessage = True

   End If

End Function

